==> MeatAxe-2.4.8.tar.gz: mtx/2.4/src/mscore.c <==
/* ============================= C MeatAxe ==================================
   File:        $Id: mscore.c,v 1.1.1.1 2000/01/07 16:42:27 mringe Exp $
   Comment:     Matrix set core functions.
   --------------------------------------------------------------------------
   (C) Copyright 1999 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"
#include <string.h>

   
/* --------------------------------------------------------------------------
   Local data
   -------------------------------------------------------------------------- */

MTX_DEFINE_FILE_INFO
#define MS_MAGIC 0x6263659B
   





/*{{{ --------------------------------------------------------------------------
!definesection obj.matset
\index{MatrixSet_t@{\tt MatrixSet\protect\_t} data type}
A |MatrixSet_t| structure is a sequence of lineary independent 
matrices $A_1,\ldots,A_l\in\GF(q)^{n\times m}$. In other words, a
|MatrixSet_t| is a basis of a subspace of $\GF(q)^{n\times m}$.

There is only one way to create a matrix set: the application first
allocates a |MatrixSet_t| structure, and then adds matrices to the 
set with |MsCleanAndAppend()|. The latter function guarantees that
the matrix set remains linearly independent --- it will not add a 
matrix which is already in the span of the set.
A second function, |MsClean()|, can be used to determine if a
matrix is in the span of a matrix set without modifying the set.


Once a matrix has been added to a matrix set, the set takes the
ownership of that matrix. The application must not modify of free 
a matrix after it has een added to a matrix set. When the matrix 
set is freed with |MsFree()|, all matrices in  the set are freed, 
too.
-------------------------------------------------------------------------- }}}*/


/*{{{ --------------------------------------------------------------------------
!section obj.matset
!function MsIsValid "Check a matrix set"
!arg set
    Pointer to the matrix set.
!return
    1 if |set| points to a valid matrix set, 0 otherwise.
!description
    This function checks if the argument |set| is a pointer to a valid
    matrix. If the matrix set is valid, the function returns 1. Otherwise, 
    an error is signalled and, if the error handler does not terminate the 
    program, the function returns 0.
!seealso MatrixSet_t
-------------------------------------------------------------------------- }}}*/

int MsIsValid(const MatrixSet_t *set)

{
#ifdef PARANOID
    int i;
    MatrixSetElement_t *l;
    int field, nor, noc;
#endif

    if (set == NULL || set->Magic != MS_MAGIC)
    {
	MTX_ERROR1("Invalid matrix set at 0x%lx",(long) set);
	return 0;
    }
    if (set->Len < 0)
    {
	MTX_ERROR1("Invalid matrix set: len=%d",set->Len);
	return 0;
    }
    if (set->Len > 0 && set->List == NULL)
    {
	MTX_ERROR("Invalid matrix set: list=NULL");
	return 0;
    }

#ifdef PARANOID
    for (i = 0, l = set->List; i < set->Len; ++i, ++l)
    {
	if (!MatIsValid(l->Matrix))
	{
	    MTX_ERROR1("Matrix %d is invalid",i);
	    return 0;
	}
	if (i == 0)
	{
	    field = l->Matrix->Field;
	    nor = l->Matrix->Nor;
	    noc = l->Matrix->Noc;
	}
	else if (field != l->Matrix->Field ||
	    nor != l->Matrix->Nor || noc != l->Matrix->Noc)
	{
	    MTX_ERROR3("Invalid matrix set: Matrix %d and %d: %E",
		i,i-1,MTX_ERR_INCOMPAT);
	    return 0;
	}
	if (l->PivRow < 0 || l->PivRow >= nor)
	{
	    MTX_ERROR2("Invalid pivot row %d, nor=%d",l->PivRow,nor);
	    return 0;
	}
	if (l->PivCol < 0 || l->PivCol >= noc)
	{
	    MTX_ERROR2("Invalid pivot column %d, noc=%d",l->PivRow,noc);
	    return 0;
	}
    }
#endif
    return 1;
}




/*{{{ --------------------------------------------------------------------------
!function MsAlloc "Allocate a matrix set"
!return
    Pointer to the new matrix set, or |NULL| on error.
!description
    This function allocates a new matrix set. The set is initially empty.
    When the set is no longer needed, if must be freed with 
    |MsFree()|, {\em not} |free()| or |SysFree()|.
!seealso MatrixSet_t MsFree
-------------------------------------------------------------------------- }}}*/

MatrixSet_t *MsAlloc()

{
    MatrixSet_t *x;

    x = ALLOC(MatrixSet_t);
    if (x == NULL)
    {
	MTX_ERROR("Cannot allocate matrix set");
	return NULL;
    }
    memset(x,0,sizeof(*x));
    x->Magic = MS_MAGIC;
    return x;
}




/*{{{ --------------------------------------------------------------------------
!function MsFree "Free a matrix set"
!arg set
    Pointer to the matrix set.
!return
    $0$ on success, $-1$ on error.
!description
    This function frees a matrix set. All matrices in the set are freed, too.
!seealso MatrixSet_t MsAlloc
-------------------------------------------------------------------------- }}}*/

int MsFree(MatrixSet_t *set)

{
    int i;
    if (!MsIsValid(set))
	return -1;
    for (i = 0; i < set->Len; ++i)
	MatFree(set->List[i].Matrix);
    SysFree(set->List);
    memset(set,0,sizeof(*set));
    return 0;
}

==> MeatAxe-2.4.9.tar.gz: mtx/2.4/src/mscore.c <==
/* ============================= C MeatAxe ==================================
   File:        $Id: mscore.c,v 1.1.1.1 2007/09/02 11:06:17 mringe Exp $
   Comment:     Matrix set core functions.
   --------------------------------------------------------------------------
   (C) Copyright 1999 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"
#include <string.h>

   
/* --------------------------------------------------------------------------
   Local data
   -------------------------------------------------------------------------- */

MTX_DEFINE_FILE_INFO
#define MS_MAGIC 0x6263659B
   





/**
!definesection obj.matset
\index{MatrixSet_t@{\tt MatrixSet\protect\_t} data type}
A |MatrixSet_t| structure is a sequence of lineary independent 
matrices $A_1,\ldots,A_l\in\GF(q)^{n\times m}$. In other words, a
|MatrixSet_t| is a basis of a subspace of $\GF(q)^{n\times m}$.

There is only one way to create a matrix set: the application first
allocates a |MatrixSet_t| structure, and then adds matrices to the 
set with |MsCleanAndAppend()|. The latter function guarantees that
the matrix set remains linearly independent --- it will not add a 
matrix which is already in the span of the set.
A second function, |MsClean()|, can be used to determine if a
matrix is in the span of a matrix set without modifying the set.


Once a matrix has been added to a matrix set, the set takes the
ownership of that matrix. The application must not modify of free 
a matrix after it has een added to a matrix set. When the matrix 
set is freed with |MsFree()|, all matrices in  the set are freed, 
too.
 **/


/**
!section obj.matset
!function MsIsValid "Check a matrix set"
 ** @param set
    Pointer to the matrix set.
 ** @return
    1 if |set| points to a valid matrix set, 0 otherwise.
!description
    This function checks if the argument |set| is a pointer to a valid
    matrix. If the matrix set is valid, the function returns 1. Otherwise, 
    an error is signalled and, if the error handler does not terminate the 
    program, the function returns 0.
 ** @see MatrixSet_t
 **/

int MsIsValid(const MatrixSet_t *set)

{
#ifdef PARANOID
    int i;
    MatrixSetElement_t *l;
    int field, nor, noc;
#endif

    if (set == NULL || set->Magic != MS_MAGIC)
    {
	MTX_ERROR1("Invalid matrix set at 0x%lx",(long) set);
	return 0;
    }
    if (set->Len < 0)
    {
	MTX_ERROR1("Invalid matrix set: len=%d",set->Len);
	return 0;
    }
    if (set->Len > 0 && set->List == NULL)
    {
	MTX_ERROR("Invalid matrix set: list=NULL");
	return 0;
    }

#ifdef PARANOID
    for (i = 0, l = set->List; i < set->Len; ++i, ++l)
    {
	if (!MatIsValid(l->Matrix))
	{
	    MTX_ERROR1("Matrix %d is invalid",i);
	    return 0;
	}
	if (i == 0)
	{
	    field = l->Matrix->Field;
	    nor = l->Matrix->Nor;
	    noc = l->Matrix->Noc;
	}
	else if (field != l->Matrix->Field ||
	    nor != l->Matrix->Nor || noc != l->Matrix->Noc)
	{
	    MTX_ERROR3("Invalid matrix set: Matrix %d and %d: %E",
		i,i-1,MTX_ERR_INCOMPAT);
	    return 0;
	}
	if (l->PivRow < 0 || l->PivRow >= nor)
	{
	    MTX_ERROR2("Invalid pivot row %d, nor=%d",l->PivRow,nor);
	    return 0;
	}
	if (l->PivCol < 0 || l->PivCol >= noc)
	{
	    MTX_ERROR2("Invalid pivot column %d, noc=%d",l->PivRow,noc);
	    return 0;
	}
    }
#endif
    return 1;
}




/**
!function MsAlloc "Allocate a matrix set"
 ** @return
    Pointer to the new matrix set, or |NULL| on error.
!description
    This function allocates a new matrix set. The set is initially empty.
    When the set is no longer needed, if must be freed with 
    |MsFree()|, {\em not} |free()| or |SysFree()|.
 ** @see MatrixSet_t MsFree
 **/

MatrixSet_t *MsAlloc()

{
    MatrixSet_t *x;

    x = ALLOC(MatrixSet_t);
    if (x == NULL)
    {
	MTX_ERROR("Cannot allocate matrix set");
	return NULL;
    }
    memset(x,0,sizeof(*x));
    x->Magic = MS_MAGIC;
    return x;
}




/**
!function MsFree "Free a matrix set"
 ** @param set
    Pointer to the matrix set.
 ** @return
    $0$ on success, $-1$ on error.
!description
    This function frees a matrix set. All matrices in the set are freed, too.
 ** @see MatrixSet_t MsAlloc
 **/

int MsFree(MatrixSet_t *set)

{
    int i;
    if (!MsIsValid(set))
	return -1;
    for (i = 0; i < set->Len; ++i)
	MatFree(set->List[i].Matrix);
    SysFree(set->List);
    memset(set,0,sizeof(*set));
    return 0;
}
